# -*- coding: utf-8 -*-
# @Author  : LG

from PyQt5 import QtCore, QtWidgets, QtGui
from IDAT.rect import Rect
from enum import Enum
from typing import List


class DRAWMode(Enum):
    VIEW = 0
    CREATE = 1
    EDIT = 2

class AnnotateMode(Enum):
    DETECT = 0
    MULTI_CLASSIFY = 1


class Scene(QtWidgets.QGraphicsScene):
    def __init__(self, mainwindow):
        super(Scene, self).__init__()
        self.mainwindow = mainwindow
        self.pixmap_item:QtWidgets.QGraphicsPixmapItem = None
        self.rects:List[Rect] = []
        self.drawmode:DRAWMode = View
        self.current_rect:Rect = None
        self.leftpressed = False

        self.guide_line_x:QtWidgets.QGraphicsLineItem = None
        self.guide_line_y:QtWidgets.QGraphicsLineItem = None

    def change_draw_to_create(self):
        self.drawmode = DRAWMode.CREATE
        self.leftpressed = False
        if self.pixmap_item is not None:
            self.pixmap_item.setCursor(QtGui.QCursor(QtCore.Qt.CursorShape.CrossCursor))
            self.mainwindow.actionCache.setEnabled(True)

    def change_draw_to_edit(self):
        self.drawmode = DRAWMode.EDIT
        self.leftpressed = False
        if self.pixmap_item is not None:
            self.pixmap_item.setCursor(QtGui.QCursor(QtCore.Qt.CursorShape.CrossCursor))

    def change_draw_to_view(self):
        self.drawmode = DRAWMode.VIEW
        self.leftpressed = False
        # 移除辅助线
        if self.guide_line_x is not None and self.guide_line_y is not None:
            if self.guide_line_x in self.items():
                self.removeItem(self.guide_line_x)
            if self.guide_line_y in self.items():
                self.removeItem(self.guide_line_y)
            self.guide_line_x = None
            self.guide_line_y = None
        if self.pixmap_item is not None:
            self.pixmap_item.setCursor(QtGui.QCursor(QtCore.Qt.CursorShape.ArrowCursor))
            self.mainwindow.actionCache.setEnabled(False)

    def load_pixmap(self, pixmap):

        self.pixmap_item = QtWidgets.QGraphicsPixmapItem()
        self.pixmap_item.setZValue(0)
        self.pixmap_item.setPixmap(pixmap)
        self.addItem(self.pixmap_item)
        self.setSceneRect(self.pixmap_item.boundingRect())
        self.change_draw_to_view()

    def start_draw(self):
        if len(self.mainwindow.listWidget_categories) < 1:
            QtWidgets.QMessageBox.warning(self.mainwindow, 'Warning', 'The category list is empty.\nSet categorys before labeling.')
            return
        if self.drawmode != DRAWMode.VIEW:
            return
        self.mainwindow.label_visiable_checkbox_click(False)
        self.change_draw_to_create()
        self.current_rect = Rect()
        self.addItem(self.current_rect)

    def finish_draw(self):
        item = self.mainwindow.listWidget_categories.currentItem()
        widget = self.mainwindow.listWidget_categories.itemWidget(item)
        label_category = widget.findChild(QtWidgets.QLabel, 'category')
        label_color = widget.findChild(QtWidgets.QLabel, 'color')
        category = label_category.text()
        color = label_color.text()

        difficult = False
        self.mainwindow.scene.current_rect.complete(category, color, difficult)
        self.mainwindow.scene.rects.append(self.mainwindow.scene.current_rect)
        self.mainwindow.dock_labels_add_label(self.mainwindow.scene.current_rect)
        self.mainwindow.scene.current_rect = None
        self.mainwindow.set_changed(True)
        self.mainwindow.label_visiable_checkbox_click(True)

        self.change_draw_to_view()

    def cache_draw(self):
        if self.current_rect is not None:
            self.removeItem(self.current_rect)
            self.current_rect:Rect = None
        self.mainwindow.label_visiable_checkbox_click(True)
        self.change_draw_to_view()

    def edit_rect(self):
        self.change_draw_to_edit()
        rects = [item for item in self.selectedItems() if item in self.rects]
        if len(rects) < 1:
            return
        rect = rects[0]
        self.mainwindow.edit_label_dialog.init(rect)
        self.mainwindow.edit_label_dialog.show()

    def delete_rect(self):
        for rect in self.selectedItems():
            if rect in self.rects and rect.isSelected():
                index = self.rects.index(rect)
                for vertex in rect.vertexs:
                    self.removeItem(vertex)
                    del vertex
                self.removeItem(rect)
                del rect
                item = self.mainwindow.listWidget_labels.item(index)
                # self.mainwindow.listWidget_labels.removeItemWidget(item)
                self.rects.pop(index)
                self.mainwindow.dock_labels_updata()
        self.mainwindow.set_changed(True)

    def mousePressEvent(self, event: 'QtWidgets.QGraphicsSceneMouseEvent'):
        # 限制范围
        pos = event.scenePos()
        if pos.x() < 0: pos.setX(0)
        if pos.x() > self.width(): pos.setX(self.width())
        if pos.y() < 0: pos.setY(0)
        if pos.y() > self.height(): pos.setY(self.height())

        if event.button() == QtCore.Qt.MouseButton.LeftButton:
            if self.drawmode == DRAWMode.CREATE:
                # scenepos = event.scenePos()
                if self.leftpressed:
                    self.current_rect.removePoint(len(self.current_rect.points) - 1)
                    self.current_rect.addPoint(pos)
                    self.finish_draw()
                    self.leftpressed = False
                else:
                    # 第一次点击
                    self.current_rect.addPoint(pos)
                    self.current_rect.addPoint(pos)
                    self.leftpressed = True
        super(Scene, self).mousePressEvent(event)

    def mouseMoveEvent(self, event: 'QtWidgets.QGraphicsSceneMouseEvent'):
        # 限制范围
        pos = event.scenePos()
        if pos.x() < 0: pos.setX(0)
        if pos.x() > self.width(): pos.setX(self.width())
        if pos.y() < 0: pos.setY(0)
        if pos.y() > self.height(): pos.setY(self.height())

        if self.drawmode == DRAWMode.CREATE:
            # 移除旧辅助线
            if self.guide_line_x is not None and self.guide_line_y is not None:
                if self.guide_line_x in self.items():
                    self.removeItem(self.guide_line_x)

                if self.guide_line_y in self.items():
                    self.removeItem(self.guide_line_y)

                self.guide_line_x = None
                self.guide_line_y = None
            # 添加新辅助线
            if self.guide_line_x is None and self.width() > 0 and self.height() > 0:
                self.guide_line_x = QtWidgets.QGraphicsLineItem(QtCore.QLineF(pos.x(), 0, pos.x(), self.height()))
                self.addItem(self.guide_line_x)
            if self.guide_line_y is None and self.width() > 0 and self.height() > 0:
                self.guide_line_y = QtWidgets.QGraphicsLineItem(QtCore.QLineF(0, pos.y(), self.width(), pos.y()))
                self.addItem(self.guide_line_y)

            self.current_rect.movePoint(len(self.current_rect.points)-1, pos)

        self.mainwindow.labelCoordinates.setText("({:.0f}, {:.0f})".format(pos.x(), pos.y()))
        super(Scene, self).mouseMoveEvent(event)


class View(QtWidgets.QGraphicsView):
    def __init__(self, parent):
        super(View, self).__init__(parent)
        self.factor = 0.8
        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
        self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarPolicy.ScrollBarAlwaysOn)
        self.setDragMode(QtWidgets.QGraphicsView.DragMode.ScrollHandDrag)
        self.setMouseTracking(True)

    def zoom(self, factor, point=None):
        mouse_old = self.mapToScene(point) if point is not None else None
        pix_widget = self.transform().scale(factor, factor).mapRect(QtCore.QRectF(0, 0, 1, 1)).width()
        if pix_widget > 10 or pix_widget < 0.01:
            return
        self.scale(factor,factor)
        if point is not None:
            mouse_now = self.mapToScene(point)
            center_now = self.mapToScene(self.viewport().width() // 2, self.viewport().height() // 2)
            center_new = mouse_old - mouse_now + center_now
            self.centerOn(center_new)

    def zoom_fit(self):
        self.fitInView(0, 0, self.scene().width(), self.scene().height(),  QtCore.Qt.AspectRatioMode.KeepAspectRatio)

    def zoom_in(self):
        self.zoom(1 / self.factor)

    def zoom_out(self):
        self.zoom(self.factor)

    def wheelEvent(self, event: QtGui.QWheelEvent):
        angel= event.angleDelta()
        if angel.y() < 0:
            self.zoom(self.factor, event.pos())
        elif angel.y() > 0:
            self.zoom(1/self.factor, event.pos())
        else:
            pass



# -*- coding: utf-8 -*-
# @Author  : LG
from collections import namedtuple
from json import dump, load
from typing import List, Dict
from xml.etree import ElementTree as ET
import os
import yaml

CATEGORYTUPLE = namedtuple('LABEL', ['category', 'color'])

def load_config(file):
    result = []
    with open(file, 'rb')as f:
        label = yaml.load(f.read(), Loader=yaml.FullLoader)['label']

    for d in label:
        category = d['name']
        color = d['color']
        label_tuple = CATEGORYTUPLE(category, color)
        result.append(label_tuple)

    return result

def save_config(labeltuple_list:List[CATEGORYTUPLE], file:str):
    cfg = {}
    cfg['label'] = []
    for cate, colo in labeltuple_list:
        cfg['label'].append({'name': cate, 'color': colo})
    s = yaml.dump(cfg)
    with open(file, 'w') as f:
        f.write(s)
    return True

def save_category_cfg(labeltuple_list:List[CATEGORYTUPLE], file:str):
    if file.endswith('.txt'):
        with open(file, 'w') as f:
            for labeltuple in labeltuple_list:
                f.write("{} {}\n".format(labeltuple.category, labeltuple.color))
        return True

    elif file.endswith('.json'):
        D = {}
        D['categorys'] = []
        for labeltuple in labeltuple_list:
            D['categorys'].append({'category': labeltuple.category,
                                   'color': labeltuple.color})
        with open(file, 'w') as f:
            dump(D, f, indent=4)

        return True
    else:
        return False

def read_category_cfg(file:str):
    result = []
    print(file)
    if file.endswith('.txt'):
        with open(file, 'r') as f:
            lines = f.readlines()
            for line in lines:
                line:str = line.rstrip('\n')
                category, color = line.split(' ')
                label_tuple = CATEGORYTUPLE(category, color)
                result.append(label_tuple)
        return result
    elif file.endswith('.json'):
        with open(file, 'r') as f:
            D = load(f)
            categorys = D['categorys']
            for category_color_dict in categorys:
                category, color = category_color_dict['category'], category_color_dict['color']
                label_tuple = CATEGORYTUPLE(category, color)
                result.append(label_tuple)
        return result
    else:
        return None

def annotation_count(label_root:str):
    record_dict = {}
    label_files = [f for f in os.listdir(label_root) if f.endswith('.xml')]
    for file in label_files:
        file = os.path.join(label_root, file)
        tree = ET.parse(file)
        objs = tree.findall('object')
        for obj in objs:
            name = obj.find('name').text
            is_difficult = bool(int(obj.find('difficult').text))
            if name not in record_dict:
                record_dict[name] = {'total_num':1, 'diff_num':int(is_difficult)}
            else:
                record_dict[name]['total_num'] += 1
                record_dict[name]['diff_num'] += int(is_difficult)
    return record_dict



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/super/PycharmProjects/IDAT/choice_label_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog_Choice_Label(object):
    def setupUi(self, Dialog_Choice_Label):
        Dialog_Choice_Label.setObjectName("Dialog_Choice_Label")
        Dialog_Choice_Label.resize(300, 350)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        Dialog_Choice_Label.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog_Choice_Label)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.listWidget_category = QtWidgets.QListWidget(Dialog_Choice_Label)
        self.listWidget_category.setObjectName("listWidget_category")
        self.verticalLayout.addWidget(self.listWidget_category)
        self.lineEdit_category = QtWidgets.QLineEdit(Dialog_Choice_Label)
        self.lineEdit_category.setReadOnly(True)
        self.lineEdit_category.setObjectName("lineEdit_category")
        self.verticalLayout.addWidget(self.lineEdit_category)
        self.widget = QtWidgets.QWidget(Dialog_Choice_Label)
        self.widget.setObjectName("widget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout.addWidget(self.widget)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.checkBox_difficult = QtWidgets.QCheckBox(Dialog_Choice_Label)
        self.checkBox_difficult.setObjectName("checkBox_difficult")
        self.horizontalLayout_3.addWidget(self.checkBox_difficult)
        self.horizontalLayout.addLayout(self.horizontalLayout_3)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.pushButton_cache = QtWidgets.QPushButton(Dialog_Choice_Label)
        self.pushButton_cache.setMinimumSize(QtCore.QSize(80, 0))
        self.pushButton_cache.setMaximumSize(QtCore.QSize(80, 16777215))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/关闭_close-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_cache.setIcon(icon)
        self.pushButton_cache.setObjectName("pushButton_cache")
        self.horizontalLayout.addWidget(self.pushButton_cache)
        self.pushButton_apply = QtWidgets.QPushButton(Dialog_Choice_Label)
        self.pushButton_apply.setMinimumSize(QtCore.QSize(80, 0))
        self.pushButton_apply.setMaximumSize(QtCore.QSize(80, 16777215))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/icons/校验_check-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_apply.setIcon(icon1)
        self.pushButton_apply.setObjectName("pushButton_apply")
        self.horizontalLayout.addWidget(self.pushButton_apply)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(Dialog_Choice_Label)
        QtCore.QMetaObject.connectSlotsByName(Dialog_Choice_Label)

    def retranslateUi(self, Dialog_Choice_Label):
        _translate = QtCore.QCoreApplication.translate
        Dialog_Choice_Label.setWindowTitle(_translate("Dialog_Choice_Label", "Choice Label"))
        self.checkBox_difficult.setText(_translate("Dialog_Choice_Label", "difficult"))
        self.pushButton_cache.setText(_translate("Dialog_Choice_Label", "cache"))
        self.pushButton_apply.setText(_translate("Dialog_Choice_Label", "apply"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/lg/PycharmProjects/IDAT/IDAT/ui/about_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog_About(object):
    def setupUi(self, Dialog_About):
        Dialog_About.setObjectName("Dialog_About")
        Dialog_About.resize(550, 250)
        Dialog_About.setMinimumSize(QtCore.QSize(550, 250))
        Dialog_About.setMaximumSize(QtCore.QSize(550, 250))
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog_About)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget = QtWidgets.QWidget(Dialog_About)
        self.widget.setObjectName("widget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.widget_2 = QtWidgets.QWidget(self.widget)
        self.widget_2.setMaximumSize(QtCore.QSize(16777215, 40))
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.widget_2)
        self.label.setMaximumSize(QtCore.QSize(16777215, 50))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(20)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout_2.addWidget(self.widget_2)
        self.widget_3 = QtWidgets.QWidget(self.widget)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.widget_3.setFont(font)
        self.widget_3.setObjectName("widget_3")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.widget_3)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_2 = QtWidgets.QLabel(self.widget_3)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(18)
        self.label_2.setFont(font)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_3.addWidget(self.label_2)
        self.verticalLayout_2.addWidget(self.widget_3)
        self.verticalLayout.addWidget(self.widget)
        self.widget_4 = QtWidgets.QWidget(Dialog_About)
        self.widget_4.setMaximumSize(QtCore.QSize(16777215, 60))
        self.widget_4.setObjectName("widget_4")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget_4)
        self.horizontalLayout_2.setContentsMargins(9, 9, 9, 9)
        self.horizontalLayout_2.setSpacing(6)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_version = QtWidgets.QLabel(self.widget_4)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_version.setFont(font)
        self.label_version.setText("")
        self.label_version.setObjectName("label_version")
        self.horizontalLayout_2.addWidget(self.label_version)
        self.label_4 = QtWidgets.QLabel(self.widget_4)
        self.label_4.setMaximumSize(QtCore.QSize(16777215, 40))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.label_4.setFont(font)
        self.label_4.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.label_4.setStyleSheet("color: rgb(133, 0, 0);")
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_2.addWidget(self.label_4)
        self.verticalLayout.addWidget(self.widget_4)

        self.retranslateUi(Dialog_About)
        QtCore.QMetaObject.connectSlotsByName(Dialog_About)

    def retranslateUi(self, Dialog_About):
        _translate = QtCore.QCoreApplication.translate
        Dialog_About.setWindowTitle(_translate("Dialog_About", "About"))
        self.label.setText(_translate("Dialog_About", "About"))
        self.label_2.setText(_translate("Dialog_About", "IDAT is a image detection annotation tool."))
        self.label_4.setText(_translate("Dialog_About", "IDAT Copyright (C) 2022 yatengLG.\n"
"http://www.yatenglg.cn/idat"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/lg/PycharmProjects/IDAT/IDAT/ui/main_window.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1200, 720)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        MainWindow.setFont(font)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1200, 29))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(10)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        self.menuFile.setFont(font)
        self.menuFile.setObjectName("menuFile")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        self.menuEdit = QtWidgets.QMenu(self.menubar)
        self.menuEdit.setObjectName("menuEdit")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuLanguage = QtWidgets.QMenu(self.menuHelp)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        self.menuLanguage.setFont(font)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/翻译_translate.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.menuLanguage.setIcon(icon)
        self.menuLanguage.setObjectName("menuLanguage")
        MainWindow.setMenuBar(self.menubar)
        self.toolBar = QtWidgets.QToolBar(MainWindow)
        self.toolBar.setOrientation(QtCore.Qt.Horizontal)
        self.toolBar.setIconSize(QtCore.QSize(24, 24))
        self.toolBar.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
        self.toolBar.setObjectName("toolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.Labels = QtWidgets.QDockWidget(MainWindow)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        self.Labels.setFont(font)
        self.Labels.setContextMenuPolicy(QtCore.Qt.DefaultContextMenu)
        self.Labels.setAcceptDrops(False)
        self.Labels.setObjectName("Labels")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout.setContentsMargins(3, 3, 3, 3)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget = QtWidgets.QWidget(self.dockWidgetContents)
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setContentsMargins(-1, 0, -1, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.checkBox_visiable = QtWidgets.QCheckBox(self.widget)
        self.checkBox_visiable.setStatusTip("")
        self.checkBox_visiable.setObjectName("checkBox_visiable")
        self.horizontalLayout.addWidget(self.checkBox_visiable)
        self.verticalLayout.addWidget(self.widget)
        self.listWidget_labels = QtWidgets.QListWidget(self.dockWidgetContents)
        self.listWidget_labels.setObjectName("listWidget_labels")
        self.verticalLayout.addWidget(self.listWidget_labels)
        self.pushButton_add_manually = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButton_add_manually.setObjectName("pushButton_add_manually")
        self.verticalLayout.addWidget(self.pushButton_add_manually)
        self.Labels.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.Labels)
        self.Files = QtWidgets.QDockWidget(MainWindow)
        self.Files.setObjectName("Files")
        self.dockWidgetContents_2 = QtWidgets.QWidget()
        self.dockWidgetContents_2.setObjectName("dockWidgetContents_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.dockWidgetContents_2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.listWidget_files = QtWidgets.QListWidget(self.dockWidgetContents_2)
        self.listWidget_files.setObjectName("listWidget_files")
        self.verticalLayout_2.addWidget(self.listWidget_files)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lineEdit_jump = QtWidgets.QLineEdit(self.dockWidgetContents_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        self.lineEdit_jump.setFont(font)
        self.lineEdit_jump.setObjectName("lineEdit_jump")
        self.horizontalLayout_2.addWidget(self.lineEdit_jump)
        self.label_files = QtWidgets.QLabel(self.dockWidgetContents_2)
        self.label_files.setMinimumSize(QtCore.QSize(50, 0))
        self.label_files.setText("")
        self.label_files.setTextFormat(QtCore.Qt.AutoText)
        self.label_files.setScaledContents(False)
        self.label_files.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_files.setObjectName("label_files")
        self.horizontalLayout_2.addWidget(self.label_files)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.Files.setWidget(self.dockWidgetContents_2)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.Files)
        self.status_bar = QtWidgets.QStatusBar(MainWindow)
        self.status_bar.setObjectName("status_bar")
        MainWindow.setStatusBar(self.status_bar)
        self.dockWidget = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents_3 = QtWidgets.QWidget()
        self.dockWidgetContents_3.setObjectName("dockWidgetContents_3")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.dockWidgetContents_3)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.listWidget_categories = QtWidgets.QListWidget(self.dockWidgetContents_3)
        self.listWidget_categories.setObjectName("listWidget_categories")
        self.verticalLayout_3.addWidget(self.listWidget_categories)
        self.dockWidget.setWidget(self.dockWidgetContents_3)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget)
        self.actionOpen_image_dir = QtWidgets.QAction(MainWindow)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/icons/照片_pic.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOpen_image_dir.setIcon(icon1)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionOpen_image_dir.setFont(font)
        self.actionOpen_image_dir.setObjectName("actionOpen_image_dir")
        self.actionOpen_label_dir = QtWidgets.QAction(MainWindow)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/icons/文件夹-开_folder-open.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOpen_label_dir.setIcon(icon2)
        self.actionOpen_label_dir.setWhatsThis("")
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionOpen_label_dir.setFont(font)
        self.actionOpen_label_dir.setObjectName("actionOpen_label_dir")
        self.actionExit = QtWidgets.QAction(MainWindow)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/icons/icons/开关_power.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionExit.setIcon(icon3)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionExit.setFont(font)
        self.actionExit.setObjectName("actionExit")
        self.actionZoom_in = QtWidgets.QAction(MainWindow)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/icons/icons/放大_zoom-in.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionZoom_in.setIcon(icon4)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionZoom_in.setFont(font)
        self.actionZoom_in.setObjectName("actionZoom_in")
        self.actionZoom_out = QtWidgets.QAction(MainWindow)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/icons/icons/缩小_zoom-out.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionZoom_out.setIcon(icon5)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionZoom_out.setFont(font)
        self.actionZoom_out.setObjectName("actionZoom_out")
        self.actionFit_window = QtWidgets.QAction(MainWindow)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/icons/icons/全宽_fullwidth.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionFit_window.setIcon(icon6)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionFit_window.setFont(font)
        self.actionFit_window.setObjectName("actionFit_window")
        self.actionCreate = QtWidgets.QAction(MainWindow)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/icons/icons/锚点_anchor.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionCreate.setIcon(icon7)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionCreate.setFont(font)
        self.actionCreate.setObjectName("actionCreate")
        self.actionEdit = QtWidgets.QAction(MainWindow)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/icons/icons/编辑文件_file-editing.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionEdit.setIcon(icon8)
        self.actionEdit.setWhatsThis("")
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionEdit.setFont(font)
        self.actionEdit.setObjectName("actionEdit")
        self.actionDelete = QtWidgets.QAction(MainWindow)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/icons/icons/删除_delete.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionDelete.setIcon(icon9)
        self.actionDelete.setWhatsThis("")
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionDelete.setFont(font)
        self.actionDelete.setObjectName("actionDelete")
        self.actionSave = QtWidgets.QAction(MainWindow)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/icons/icons/保存_save.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSave.setIcon(icon10)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionSave.setFont(font)
        self.actionSave.setObjectName("actionSave")
        self.actionPrior_image = QtWidgets.QAction(MainWindow)
        icon11 = QtGui.QIcon()
        icon11.addPixmap(QtGui.QPixmap(":/icons/icons/上一步_back.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionPrior_image.setIcon(icon11)
        self.actionPrior_image.setStatusTip("")
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionPrior_image.setFont(font)
        self.actionPrior_image.setObjectName("actionPrior_image")
        self.actionNext_image = QtWidgets.QAction(MainWindow)
        icon12 = QtGui.QIcon()
        icon12.addPixmap(QtGui.QPixmap(":/icons/icons/下一步_next.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionNext_image.setIcon(icon12)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionNext_image.setFont(font)
        self.actionNext_image.setObjectName("actionNext_image")
        self.actionSetting = QtWidgets.QAction(MainWindow)
        icon13 = QtGui.QIcon()
        icon13.addPixmap(QtGui.QPixmap(":/icons/icons/设置_setting-two.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSetting.setIcon(icon13)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionSetting.setFont(font)
        self.actionSetting.setObjectName("actionSetting")
        self.actionCache = QtWidgets.QAction(MainWindow)
        icon14 = QtGui.QIcon()
        icon14.addPixmap(QtGui.QPixmap(":/icons/icons/关闭_close-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionCache.setIcon(icon14)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionCache.setFont(font)
        self.actionCache.setObjectName("actionCache")
        self.actionShortcut = QtWidgets.QAction(MainWindow)
        icon15 = QtGui.QIcon()
        icon15.addPixmap(QtGui.QPixmap(":/icons/icons/键盘_keyboard-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShortcut.setIcon(icon15)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionShortcut.setFont(font)
        self.actionShortcut.setObjectName("actionShortcut")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        icon16 = QtGui.QIcon()
        icon16.addPixmap(QtGui.QPixmap(":/icons/icons/我的_me.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAbout.setIcon(icon16)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionAbout.setFont(font)
        self.actionAbout.setObjectName("actionAbout")
        self.actionChinese = QtWidgets.QAction(MainWindow)
        self.actionChinese.setObjectName("actionChinese")
        self.actionEnglish = QtWidgets.QAction(MainWindow)
        self.actionEnglish.setObjectName("actionEnglish")
        self.actionVisible = QtWidgets.QAction(MainWindow)
        self.actionVisible.setCheckable(True)
        icon17 = QtGui.QIcon()
        icon17.addPixmap(QtGui.QPixmap(":/icons/icons/眼睛_eyes.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionVisible.setIcon(icon17)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.actionVisible.setFont(font)
        self.actionVisible.setObjectName("actionVisible")
        self.menuFile.addAction(self.actionOpen_image_dir)
        self.menuFile.addAction(self.actionOpen_label_dir)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionPrior_image)
        self.menuFile.addAction(self.actionNext_image)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionSetting)
        self.menuFile.addAction(self.actionExit)
        self.menuView.addAction(self.actionFit_window)
        self.menuView.addAction(self.actionZoom_in)
        self.menuView.addAction(self.actionZoom_out)
        self.menuView.addAction(self.actionVisible)
        self.menuEdit.addAction(self.actionCreate)
        self.menuEdit.addAction(self.actionCache)
        self.menuEdit.addAction(self.actionEdit)
        self.menuEdit.addAction(self.actionDelete)
        self.menuEdit.addAction(self.actionSave)
        self.menuLanguage.addAction(self.actionChinese)
        self.menuLanguage.addAction(self.actionEnglish)
        self.menuHelp.addAction(self.menuLanguage.menuAction())
        self.menuHelp.addAction(self.actionShortcut)
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())
        self.toolBar.addAction(self.actionOpen_image_dir)
        self.toolBar.addAction(self.actionOpen_label_dir)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionPrior_image)
        self.toolBar.addAction(self.actionNext_image)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionCreate)
        self.toolBar.addAction(self.actionEdit)
        self.toolBar.addAction(self.actionDelete)
        self.toolBar.addAction(self.actionSave)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionFit_window)
        self.toolBar.addAction(self.actionZoom_in)
        self.toolBar.addAction(self.actionZoom_out)
        self.toolBar.addAction(self.actionVisible)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuView.setTitle(_translate("MainWindow", "View"))
        self.menuEdit.setTitle(_translate("MainWindow", "Edit"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.menuLanguage.setTitle(_translate("MainWindow", "Language"))
        self.toolBar.setWindowTitle(_translate("MainWindow", "toolBar"))
        self.Labels.setWindowTitle(_translate("MainWindow", "Labels Dock"))
        self.checkBox_visiable.setText(_translate("MainWindow", "Visiable"))
        self.pushButton_add_manually.setStatusTip(_translate("MainWindow", "Add annotation manually."))
        self.pushButton_add_manually.setText(_translate("MainWindow", "Add Manually"))
        self.Files.setWindowTitle(_translate("MainWindow", "Files Dock"))
        self.lineEdit_jump.setStatusTip(_translate("MainWindow", "Jump to the image. Input name or index."))
        self.lineEdit_jump.setPlaceholderText(_translate("MainWindow", "Jump to the image."))
        self.dockWidget.setWindowTitle(_translate("MainWindow", "Categories Dock"))
        self.actionOpen_image_dir.setText(_translate("MainWindow", "Image dir"))
        self.actionOpen_image_dir.setStatusTip(_translate("MainWindow", "Open image directory."))
        self.actionOpen_label_dir.setText(_translate("MainWindow", "Label dir"))
        self.actionOpen_label_dir.setStatusTip(_translate("MainWindow", "Open xml file directory. This directory will be used to save xml."))
        self.actionExit.setText(_translate("MainWindow", "Exit"))
        self.actionZoom_in.setText(_translate("MainWindow", "Zoom in "))
        self.actionZoom_out.setText(_translate("MainWindow", "Zoom out"))
        self.actionFit_window.setText(_translate("MainWindow", "Fit window"))
        self.actionFit_window.setStatusTip(_translate("MainWindow", "Resize image to fit window."))
        self.actionFit_window.setShortcut(_translate("MainWindow", "F"))
        self.actionCreate.setText(_translate("MainWindow", "Create"))
        self.actionCreate.setStatusTip(_translate("MainWindow", "Create annotation rectangle."))
        self.actionCreate.setShortcut(_translate("MainWindow", "C"))
        self.actionEdit.setText(_translate("MainWindow", "Edit"))
        self.actionEdit.setStatusTip(_translate("MainWindow", "Edit the selected annotation."))
        self.actionEdit.setShortcut(_translate("MainWindow", "E"))
        self.actionDelete.setText(_translate("MainWindow", "Delete"))
        self.actionDelete.setStatusTip(_translate("MainWindow", "Delect the selected annotation."))
        self.actionDelete.setShortcut(_translate("MainWindow", "Del"))
        self.actionSave.setText(_translate("MainWindow", "Save"))
        self.actionSave.setStatusTip(_translate("MainWindow", "Save current annotation to xml file."))
        self.actionSave.setShortcut(_translate("MainWindow", "S"))
        self.actionPrior_image.setText(_translate("MainWindow", "Prior image"))
        self.actionPrior_image.setShortcut(_translate("MainWindow", "A"))
        self.actionNext_image.setText(_translate("MainWindow", "Next image"))
        self.actionNext_image.setShortcut(_translate("MainWindow", "D"))
        self.actionSetting.setText(_translate("MainWindow", "Setting"))
        self.actionCache.setText(_translate("MainWindow", "Cache"))
        self.actionCache.setShortcut(_translate("MainWindow", "Esc"))
        self.actionShortcut.setText(_translate("MainWindow", "Shortcut"))
        self.actionAbout.setText(_translate("MainWindow", "About"))
        self.actionChinese.setText(_translate("MainWindow", "中文"))
        self.actionEnglish.setText(_translate("MainWindow", "English"))
        self.actionVisible.setText(_translate("MainWindow", "Visible"))
        self.actionVisible.setShortcut(_translate("MainWindow", "V"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/super/PycharmProjects/IDAT/add_annotation_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_dialog_add_annotation(object):
    def setupUi(self, dialog_add_annotation):
        dialog_add_annotation.setObjectName("dialog_add_annotation")
        dialog_add_annotation.resize(300, 379)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        dialog_add_annotation.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(dialog_add_annotation)
        self.verticalLayout.setObjectName("verticalLayout")
        self.listWidget_category = QtWidgets.QListWidget(dialog_add_annotation)
        self.listWidget_category.setObjectName("listWidget_category")
        self.verticalLayout.addWidget(self.listWidget_category)
        self.lineEdit_category = QtWidgets.QLineEdit(dialog_add_annotation)
        self.lineEdit_category.setReadOnly(True)
        self.lineEdit_category.setObjectName("lineEdit_category")
        self.verticalLayout.addWidget(self.lineEdit_category)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lineEdit_xmin = QtWidgets.QLineEdit(dialog_add_annotation)
        self.lineEdit_xmin.setInputMethodHints(QtCore.Qt.ImhDigitsOnly)
        self.lineEdit_xmin.setObjectName("lineEdit_xmin")
        self.horizontalLayout.addWidget(self.lineEdit_xmin)
        self.lineEdit_ymin = QtWidgets.QLineEdit(dialog_add_annotation)
        self.lineEdit_ymin.setInputMethodHints(QtCore.Qt.ImhDigitsOnly)
        self.lineEdit_ymin.setObjectName("lineEdit_ymin")
        self.horizontalLayout.addWidget(self.lineEdit_ymin)
        self.lineEdit_xmax = QtWidgets.QLineEdit(dialog_add_annotation)
        self.lineEdit_xmax.setInputMethodHints(QtCore.Qt.ImhDigitsOnly)
        self.lineEdit_xmax.setObjectName("lineEdit_xmax")
        self.horizontalLayout.addWidget(self.lineEdit_xmax)
        self.lineEdit_ymax = QtWidgets.QLineEdit(dialog_add_annotation)
        self.lineEdit_ymax.setInputMethodHints(QtCore.Qt.ImhDigitsOnly)
        self.lineEdit_ymax.setCursorMoveStyle(QtCore.Qt.LogicalMoveStyle)
        self.lineEdit_ymax.setObjectName("lineEdit_ymax")
        self.horizontalLayout.addWidget(self.lineEdit_ymax)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.checkBox_difficult = QtWidgets.QCheckBox(dialog_add_annotation)
        self.checkBox_difficult.setMinimumSize(QtCore.QSize(0, 0))
        self.checkBox_difficult.setMaximumSize(QtCore.QSize(100, 16777215))
        self.checkBox_difficult.setObjectName("checkBox_difficult")
        self.horizontalLayout_2.addWidget(self.checkBox_difficult)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.pushButton_cache = QtWidgets.QPushButton(dialog_add_annotation)
        self.pushButton_cache.setMinimumSize(QtCore.QSize(80, 0))
        self.pushButton_cache.setMaximumSize(QtCore.QSize(80, 16777215))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/关闭_close-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_cache.setIcon(icon)
        self.pushButton_cache.setObjectName("pushButton_cache")
        self.horizontalLayout_2.addWidget(self.pushButton_cache)
        self.pushButton_apply = QtWidgets.QPushButton(dialog_add_annotation)
        self.pushButton_apply.setMinimumSize(QtCore.QSize(80, 0))
        self.pushButton_apply.setMaximumSize(QtCore.QSize(80, 16777215))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/icons/校验_check-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_apply.setIcon(icon1)
        self.pushButton_apply.setObjectName("pushButton_apply")
        self.horizontalLayout_2.addWidget(self.pushButton_apply)
        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(dialog_add_annotation)
        QtCore.QMetaObject.connectSlotsByName(dialog_add_annotation)

    def retranslateUi(self, dialog_add_annotation):
        _translate = QtCore.QCoreApplication.translate
        dialog_add_annotation.setWindowTitle(_translate("dialog_add_annotation", "Add annotation manually"))
        self.lineEdit_xmin.setPlaceholderText(_translate("dialog_add_annotation", "xmin"))
        self.lineEdit_ymin.setPlaceholderText(_translate("dialog_add_annotation", "ymin"))
        self.lineEdit_xmax.setPlaceholderText(_translate("dialog_add_annotation", "xmax"))
        self.lineEdit_ymax.setPlaceholderText(_translate("dialog_add_annotation", "ymax"))
        self.checkBox_difficult.setText(_translate("dialog_add_annotation", "difficult"))
        self.pushButton_cache.setText(_translate("dialog_add_annotation", "cache"))
        self.pushButton_apply.setText(_translate("dialog_add_annotation", "apply"))



# -*- coding: utf-8 -*-
# @Author  : LG

from IDAT.ui.main_window import Ui_MainWindow
from IDAT.ui.choice_label_dialog import Ui_Dialog_Choice_Label
from IDAT.ui.add_annotation_dialog import Ui_dialog_add_annotation
from IDAT.ui.setting_dialog import Ui_Dialog_Setting
from IDAT.ui.shourcut_dialog import Ui_Dialog_Shortcut
from IDAT.ui.about_dialog import Ui_Dialog_About
from IDAT.canvas import Scene, View
from PyQt5 import QtCore, QtWidgets, QtGui
from IDAT.utils.file_ops import CATEGORYTUPLE, save_config, load_config
from IDAT.anno import Annotations, Annotation
from IDAT.rect import Rect
import imgviz
from xml.etree import ElementTree as ET
import sys
import os
from IDAT import icon_rc

cmap = imgviz.label_colormap()
IDAT_ROOT = os.path.split(os.path.abspath(__file__))[0]

class SettingDialog(QtWidgets.QDialog, Ui_Dialog_Setting):
    def __init__(self, parent, mainwindow):
        super(SettingDialog, self).__init__(parent)
        self.setupUi(self)
        self.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)
        self.mainwindow = mainwindow
        self.label_tuple:list = []
        self.config_file:str = None
        self.connect()

    def init(self):
        label_tuples = self.mainwindow.category_tuples
        if label_tuples is None:
            return
        self.listWidget_labels.clear()
        self.lineEdit_label.clear()
        for label_tuple in label_tuples:
            category = label_tuple.category
            color = label_tuple.color
            item, widget = self.item_widget_generate(category, color)
            self.listWidget_labels.addItem(item)
            self.listWidget_labels.setItemWidget(item, widget)

    def button_choice_color(self):
        button = self.sender()
        color = QtWidgets.QColorDialog.getColor()
        if color.isValid():
            button.setStyleSheet('QWidget {background-color: %s}' % (color.name()))

    def delete_category_item(self):
        button = self.sender()
        row = self.listWidget_labels.indexAt(button.parent().pos()).row()
        self.listWidget_labels.takeItem(row)
        if row <= len(self.mainwindow.category_tuples)-1:
            self.mainwindow.category_tuples.pop(row)

    def item_widget_generate(self, category, color):
        item = QtWidgets.QListWidgetItem()
        item.setSizeHint(QtCore.QSize(200, 50))
        widget = QtWidgets.QWidget()

        layout = QtWidgets.QHBoxLayout()
        label_category = QtWidgets.QLabel()
        label_category.setText(category)
        label_category.setObjectName('category')

        button_color = QtWidgets.QPushButton()
        button_color.setStyleSheet('QWidget {background-color: %s}' % color)
        button_color.setFixedWidth(50)
        button_color.clicked.connect(self.button_choice_color)
        button_color.setObjectName('color')

        button_delete = QtWidgets.QPushButton()
        button_delete.setText('delete')
        button_delete.setFixedWidth(80)
        button_delete.clicked.connect(self.delete_category_item)
        button_delete.setObjectName('delete')

        layout.addWidget(label_category)
        layout.addWidget(button_color)
        layout.addWidget(button_delete)
        widget.setLayout(layout)
        return item, widget

    def load_cfg(self, file=None):
        if file is not None:
            # label_tuples = read_category_cfg(file)
            label_tuples = load_config(file)
            self.config_file = file

            self.mainwindow.actionSetting.setStatusTip(
                QtCore.QCoreApplication.translate("MainWindow", "Current config file: {}".format(self.config_file)))

        else:
            label_tuples = self.mainwindow.category_tuples
        if label_tuples is None:
            return
        self.listWidget_labels.clear()
        for label_tuple in label_tuples:
            category = label_tuple.category
            color = label_tuple.color
            item, widget = self.item_widget_generate(category, color)
            self.listWidget_labels.addItem(item)
            self.listWidget_labels.setItemWidget(item, widget)
        self.mainwindow.category_tuples = label_tuples
        return True

    def save_cfg(self, file):
        # save_category_cfg(self.mainwindow.category_tuples, file)
        save_config(self.mainwindow.category_tuples, file)
        return True

    def add(self):
        category = self.lineEdit_label.text()
        color = self.pushButton_color.palette().button().color().name()
        if category == '':
            QtWidgets.QMessageBox.warning(self, 'Warning', 'The category is empty.')
            return

        item, widget = self.item_widget_generate(category, color)
        self.listWidget_labels.addItem(item)
        self.listWidget_labels.setItemWidget(item, widget)

    def apply(self):
        self.mainwindow.category_tuples = []
        for index in range(self.listWidget_labels.count()):
            item = self.listWidget_labels.item(index)
            widget = self.listWidget_labels.itemWidget(item)
            label_category = widget.findChild(QtWidgets.QLabel, 'category')
            button_color = widget.findChild(QtWidgets.QPushButton, 'color')
            category = label_category.text()
            color = button_color.palette().button().color().name()
            self.mainwindow.category_tuples.append(CATEGORYTUPLE(category, color))
        if self.config_file is not None:
            self.save_cfg(self.config_file)
        self.mainwindow.load_category()

        r = QtWidgets.QMessageBox.information(self, 'Setting apply', 'OK!')
        if r:
            self.close()

    def cache(self):
        self.lineEdit_label.clear()
        self.pushButton_color.setStyleSheet('QWidget {background-color: #8ae234}')
        self.close()

    def export_cfg(self):
        file, suffix = QtWidgets.QFileDialog.getSaveFileName(self, caption='Save file path', filter=".yaml (*.yaml)")
        if file:
            if not file.endswith('.yaml'):
                file += '.yaml'
            if file:
                self.save_cfg(file)

    def import_cfg(self):
        file, suffix = QtWidgets.QFileDialog.getOpenFileName(self, caption='file path', filter=".yaml (*.yaml)")
        if file:
            self.load_cfg(file)

    def connect(self):
        self.pushButton_add.clicked.connect(self.add)
        self.pushButton_color.clicked.connect(self.button_choice_color)
        self.pushButton_cache.clicked.connect(self.cache)
        self.pushButton_apply.clicked.connect(self.apply)
        self.pushButton_import.clicked.connect(self.import_cfg)
        self.pushButton_export.clicked.connect(self.export_cfg)

class ShortcutDialog(QtWidgets.QDialog, Ui_Dialog_Shortcut):
    def __init__(self, parent):
        super(ShortcutDialog, self).__init__(parent)
        self.setupUi(self)
        self.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)

class AboutDialog(QtWidgets.QDialog, Ui_Dialog_About):
    def __init__(self, parent):
        super(AboutDialog, self).__init__(parent)
        self.setupUi(self)
        self.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)
        try:
            from IDAT import __version__
            self.label_version.setText('v'+ __version__)
        except: pass

class ChoiceLabelDialog(QtWidgets.QDialog, Ui_Dialog_Choice_Label):
    def __init__(self, parent, mainwindow):
        super(ChoiceLabelDialog, self).__init__(parent)
        self.setupUi(self)
        self.mainwindow = mainwindow
        self.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)

        self.current_category_tuple:CATEGORYTUPLE = None
        self.connect()

    def init(self):
        self.current_category_tuple = None
        self.lineEdit_category.clear()
        self.lineEdit_category.setStyleSheet('QWidget {background-color: #FFFFFF}')
        self.checkBox_difficult.setChecked(False)
        self.listWidget_category.clear()

        for category_tuple in self.mainwindow.category_tuples:
            category = category_tuple.category
            color = category_tuple.color
            item = QtWidgets.QListWidgetItem()
            item.setSizeHint(QtCore.QSize(200, 30))

            widget = QtWidgets.QWidget()
            layout = QtWidgets.QHBoxLayout()
            layout.setContentsMargins(9, 1, 9, 1)

            label_category = QtWidgets.QLabel()
            label_category.setText(category)

            label_color = QtWidgets.QLabel()
            label_color.setFixedWidth(10)
            label_color.setStyleSheet("background-color: {};".format(color))

            layout.addWidget(label_color)
            layout.addWidget(label_category)
            widget.setLayout(layout)

            self.listWidget_category.addItem(item)
            self.listWidget_category.setItemWidget(item, widget)

    def choice_category(self):
        current_row = self.listWidget_category.currentRow()
        category_tuple = self.mainwindow.category_tuples[current_row]
        self.lineEdit_category.setText(category_tuple.category)
        self.lineEdit_category.setStyleSheet("border-width: 2px;border-style: solid; border-color: %s;" % category_tuple.color)
        self.current_category_tuple = category_tuple

    def apply(self):
        if self.current_category_tuple is None:
            QtWidgets.QMessageBox.warning(self, 'Warning', 'No category selected.')
            return
        category = self.current_category_tuple.category
        color = self.current_category_tuple.color
        difficult = self.checkBox_difficult.checkState()
        self.mainwindow.scene.current_rect.complete(category, color, difficult)
        self.mainwindow.scene.rects.append(self.mainwindow.scene.current_rect)
        self.mainwindow.dock_labels_add_label(self.mainwindow.scene.current_rect)
        self.mainwindow.scene.current_rect = None
        self.mainwindow.set_changed(True)
        self.close()

    def cache(self):
        self.mainwindow.scene.cache_draw()
        self.close()

    def closeEvent(self, a0: QtGui.QCloseEvent):
        self.cache()

    def reject(self):
        self.cache()

    def connect(self):
        self.pushButton_apply.clicked.connect(self.apply)
        self.pushButton_cache.clicked.connect(self.cache)
        self.listWidget_category.itemClicked.connect(self.choice_category)

class EditLabelDialog(QtWidgets.QDialog, Ui_Dialog_Choice_Label):
    def __init__(self, parent, mainwindow):
        super(EditLabelDialog, self).__init__(parent)
        self.setupUi(self)
        self.mainwindow = mainwindow
        self.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)
        self.rect:Rect =  None
        self.connect()

    def init(self, rect:Rect):
        self.rect = rect
        self.rect.is_completed = False
        self.current_category_tuple = None
        self.lineEdit_category.setText(rect.category)
        self.lineEdit_category.setStyleSheet("border-width: 2px;border-style: solid; border-color: %s;" % rect.color.name())

        self.checkBox_difficult.setChecked(rect.is_difficult)
        self.listWidget_category.clear()

        for category_tuple in self.mainwindow.category_tuples:
            category = category_tuple.category
            color = category_tuple.color
            item = QtWidgets.QListWidgetItem()
            item.setSizeHint(QtCore.QSize(200, 30))

            widget = QtWidgets.QWidget()
            layout = QtWidgets.QHBoxLayout()
            layout.setContentsMargins(9, 1, 9, 1)

            label_category = QtWidgets.QLabel()
            label_category.setText(category)

            label_color = QtWidgets.QLabel()
            label_color.setFixedWidth(10)
            label_color.setStyleSheet("background-color: {};".format(color))

            layout.addWidget(label_color)
            layout.addWidget(label_category)
            widget.setLayout(layout)

            # item.setText(category)
            self.listWidget_category.addItem(item)
            self.listWidget_category.setItemWidget(item, widget)
            if category == rect.category and color == rect.color.name():
                self.current_category_tuple = category_tuple

    def choice_category(self):
        current_row = self.listWidget_category.currentRow()
        category_tuple = self.mainwindow.category_tuples[current_row]
        self.lineEdit_category.setText(category_tuple.category)
        self.lineEdit_category.setStyleSheet("border-width: 2px;border-style: solid; border-color: %s;" % category_tuple.color)
        self.current_category_tuple = category_tuple

    def apply(self):
        if self.rect is None:
            return
        if self.current_category_tuple is None:
            QtWidgets.QMessageBox.warning(self, 'Warning', 'No category selected.')
            return
        category = self.current_category_tuple.category
        color = self.current_category_tuple.color
        difficult = bool(self.checkBox_difficult.checkState())

        if category != self.rect.category or difficult != self.rect.is_difficult:
            self.mainwindow.set_changed(True)

        self.rect.category = category
        self.rect.is_difficult = difficult
        self.rect.color = QtGui.QColor(color)
        self.rect.setPen(QtGui.QPen(self.rect.color, 1, QtCore.Qt.PenStyle.SolidLine, QtCore.Qt.PenCapStyle.RoundCap))
        self.rect.color.setAlpha(self.rect.nohover_alpha)
        self.rect.setBrush(self.rect.color)
        self.rect.redraw()
        self.rect.is_completed = True
        self.rect = None
        self.mainwindow.dock_labels_updata()
        self.mainwindow.scene.change_draw_to_view()
        self.close()

    def cache(self):
        if self.rect is not None:
            self.rect.is_completed = True
            self.rect = None
        self.mainwindow.scene.change_draw_to_view()
        self.close()

    def closeEvent(self, a0: QtGui.QCloseEvent):
        self.cache()

    def reject(self):
        self.cache()

    def connect(self):
        self.pushButton_apply.clicked.connect(self.apply)
        self.pushButton_cache.clicked.connect(self.cache)
        self.listWidget_category.itemClicked.connect(self.choice_category)

class AddAnnotDialog(QtWidgets.QDialog, Ui_dialog_add_annotation):
    def __init__(self, parent, mainwindow):
        super(AddAnnotDialog, self).__init__(parent)
        self.mainwindow = mainwindow
        self.setupUi(self)
        self.setWindowModality(QtCore.Qt.WindowModality.ApplicationModal)
        self.listWidget_category.itemClicked.connect(self.choice_category)
        self.pushButton_apply.clicked.connect(self.apply)
        # self.pushButton_apply.setShortcut('Enter')
        self.pushButton_cache.clicked.connect(self.cache)
        # self.pushButton_cache.setShortcut('Esc')

    def init(self):
        self.lineEdit_category.clear()
        self.lineEdit_category.setStyleSheet('QWidget {background-color: #FFFFFF}')
        self.current_category_tuple = None
        self.lineEdit_xmin.clear()
        self.lineEdit_ymin.clear()
        self.lineEdit_xmax.clear()
        self.lineEdit_ymax.clear()
        self.checkBox_difficult.setChecked(False)
        self.listWidget_category.clear()

        for category_tuple in self.mainwindow.category_tuples:
            category = category_tuple.category
            color = category_tuple.color
            item = QtWidgets.QListWidgetItem()
            item.setSizeHint(QtCore.QSize(200, 30))

            widget = QtWidgets.QWidget()
            layout = QtWidgets.QHBoxLayout()
            layout.setContentsMargins(9, 1, 9, 1)

            label_category = QtWidgets.QLabel()
            label_category.setText(category)

            label_color = QtWidgets.QLabel()
            label_color.setFixedWidth(10)
            label_color.setStyleSheet("background-color: {};".format(color))

            layout.addWidget(label_color)
            layout.addWidget(label_category)
            widget.setLayout(layout)

            self.listWidget_category.addItem(item)
            self.listWidget_category.setItemWidget(item, widget)

    def choice_category(self):
        current_row = self.listWidget_category.currentRow()
        category_tuple = self.mainwindow.category_tuples[current_row]
        self.lineEdit_category.setText(category_tuple.category)
        self.lineEdit_category.setStyleSheet("border-width: 2px;border-style: solid; border-color: %s;" % category_tuple.color)
        self.current_category_tuple = category_tuple

    def apply(self):
        try:
            if self.current_category_tuple is None:
                QtWidgets.QMessageBox.warning(self.mainwindow, 'Warning', 'No category selected!')
                return
            xmin = int(self.lineEdit_xmin.text())
            ymin = int(self.lineEdit_ymin.text())
            xmax = int(self.lineEdit_xmax.text())
            ymax = int(self.lineEdit_ymax.text())
            is_difficult = self.checkBox_difficult.isChecked()
            category = self.current_category_tuple.category
            color = self.current_category_tuple.color

            # 添加矩形
            rect = Rect()
            self.mainwindow.scene.addItem(rect)
            rect.create_from_xyxy(category, xmin, ymin, xmax, ymax, is_difficult, color)
            self.mainwindow.scene.rects.append(rect)
            self.mainwindow.dock_labels_add_label(rect)
            self.mainwindow.set_changed(True)
            self.close()

        except:
            QtWidgets.QMessageBox.warning(self.mainwindow, 'Warning', 'Data error!')
            return

    def cache(self):
        self.close()

class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()
        self.setupUi(self)

        self.scene = Scene(self)
        self.view = View(self)
        self.view.setScene(self.scene)
        self.setCentralWidget(self.view)

        self.labelCoordinates = QtWidgets.QLabel('')
        self.status_bar.addPermanentWidget(self.labelCoordinates)
        self.setting_dialog = SettingDialog(self, self)
        self.shortcut_dialog = ShortcutDialog(self)
        self.about_dialog = AboutDialog(self)
        self.choice_label_dialog = ChoiceLabelDialog(self, self)
        self.edit_label_dialog = EditLabelDialog(self, self)
        self.add_annotation_dialog = AddAnnotDialog(self, self)

        self.files_root = None
        self.labels_root = None
        self.files_list = []
        self.current_index = 0
        self.pixmap:QtGui.QPixmap = None
        self.xml_path:str = None
        self.category_tuples = []
        self.is_changed = False

        self.trans = QtCore.QTranslator()

        self.init_gui()
        self.connect()

        # 默认类别配置
        default_yaml_path = os.path.join(IDAT_ROOT, 'default.yaml')
        if os.path.exists(default_yaml_path):
            self.setting_dialog.load_cfg(default_yaml_path)

        self.load_category()

    def init_gui(self):
        self.setWindowTitle('IDAT')
        self.actionPrior_image.setEnabled(False)
        self.actionNext_image.setEnabled(False)
        self.actionCreate.setEnabled(False)
        self.actionCache.setEnabled(False)
        self.actionEdit.setEnabled(False)
        self.actionDelete.setEnabled(False)
        self.actionSave.setEnabled(False)
        self.actionZoom_in.setEnabled(False)
        self.actionZoom_out.setEnabled(False)
        self.actionFit_window.setEnabled(False)

    def translate(self, language='zh'):
        if language == 'zh':
            self.trans.load(os.path.join(IDAT_ROOT, 'ui/zh_CN'))
        else:
            self.trans.load(os.path.join(IDAT_ROOT, 'ui/en'))
        self.actionChinese.setChecked(language=='zh')
        self.actionEnglish.setChecked(language=='en')
        _app = QtWidgets.QApplication.instance()
        _app.installTranslator(self.trans)
        self.retranslateUi(self)
        self.about_dialog.retranslateUi(self.about_dialog)
        self.add_annotation_dialog.retranslateUi(self.add_annotation_dialog)
        self.choice_label_dialog.retranslateUi(self.choice_label_dialog)
        self.setting_dialog.retranslateUi(self.setting_dialog)
        self.shortcut_dialog.retranslateUi(self.shortcut_dialog)
        self.edit_label_dialog.retranslateUi(self.edit_label_dialog)

    def translate_to_chinese(self):
        self.translate('zh')

    def translate_to_english(self):
        self.translate('en')

    def load_category(self):
        self.listWidget_categories.clear()
        label_tuples = self.category_tuples
        if label_tuples is None:
            return

        for label_tuple in label_tuples:
            category = label_tuple.category
            color = label_tuple.color

            item = QtWidgets.QListWidgetItem()
            item.setSizeHint(QtCore.QSize(200, 40))
            widget = QtWidgets.QWidget()

            layout = QtWidgets.QHBoxLayout()
            layout.setContentsMargins(9, 1, 9, 1)

            label_category = QtWidgets.QLabel()
            label_category.setText(category)
            label_category.setObjectName('category')

            button_color = QtWidgets.QLabel()
            button_color.setStyleSheet('QWidget {background-color: %s}' % color)
            button_color.setFixedWidth(5)

            label_color = QtWidgets.QLabel()
            label_color.setText(color)
            label_color.setVisible(False)
            label_color.setObjectName('color')

            layout.addWidget(label_color)
            layout.addWidget(button_color)
            layout.addWidget(label_category)
            widget.setLayout(layout)

            self.listWidget_categories.addItem(item)
            self.listWidget_categories.setItemWidget(item, widget)
        if len(label_tuples) > 0:
            self.listWidget_categories.setCurrentRow(0)

    def open_image_dir(self):
        dir = QtWidgets.QFileDialog.getExistingDirectory(self, caption='Open image directory')
        if not dir:
            return
        suffixs = ('jpg', 'JPG', 'jpeg', 'JPEG', 'png', 'PNG', 'ppm', 'PPM')
        files_list = [f for f in os.listdir(dir) if f.endswith(suffixs)]
        if len(files_list) < 1:
            QtWidgets.QMessageBox.warning(self, 'Warning', 'No pictures found!\nOnly pictures suffix as [{}] are supported!'.format(suffixs))
            return
        files_list.sort()
        self.files_list = files_list

        self.files_root = dir
        self.actionOpen_image_dir.setStatusTip(
            QtCore.QCoreApplication.translate("MainWindow", "Current image directory: {}".format(self.files_root)))

        self.labels_root = dir
        self.actionOpen_label_dir.setStatusTip(
            QtCore.QCoreApplication.translate("MainWindow", "Current label directory: {}".format(self.labels_root)))

        self.load_category_yaml_from_label_root()

        self.current_index = 0
        self.show_image(0)

        # update files dock
        self.listWidget_files.clear()
        for file_name in self.files_list:
            item = QtWidgets.QListWidgetItem()
            item.setText(file_name)
            self.listWidget_files.addItem(item)

    def open_label_dir(self):
        dir = QtWidgets.QFileDialog.getExistingDirectory(self, caption='Open label directory')
        if dir:
            self.labels_root = dir
            self.actionOpen_label_dir.setStatusTip(
                QtCore.QCoreApplication.translate("MainWindow", "Current label directory: {}".format(self.labels_root)))

            self.load_category_yaml_from_label_root()

        if self.files_list:
            self.show_image()

    def load_category_yaml_from_label_root(self):
        category_yaml = os.path.join(self.labels_root, 'category.yaml')
        if os.path.exists(category_yaml):
            self.setting_dialog.load_cfg(category_yaml)
            self.load_category()
        else:
            object_names = self.get_object_name_from_xml(self.labels_root)
            category_list = []
            if len(object_names) > 0:
                for index, object_name in enumerate(object_names):
                    r, g, b = cmap[index+1]
                    category_list.append(CATEGORYTUPLE(object_name, QtGui.QColor(r, g, b).name()))
                save_config(category_list, file=category_yaml)
                self.setting_dialog.load_cfg(category_yaml)
                self.load_category()

    def get_object_name_from_xml(self, xml_root):
        object_name_set = set()
        for xml in os.listdir(xml_root):
            try:
                xml_path = os.path.join(xml_root, xml)
                root = ET.parse(xml_path).getroot()
                objects = root.findall('object')
                for object in objects:
                    name = object.find('name').text
                    object_name_set.add(name)
            except: pass
        object_name_set = list(object_name_set)
        object_name_set.sort()
        return object_name_set

    def show_image(self, index:int=None):
        self.scene.clear()
        self.scene.rects.clear()
        self.set_changed(False)
        self.checkBox_visiable.setChecked(True)
        self.actionVisible.setChecked(True)
        self.listWidget_labels.clear()

        if len(self.files_list) == 0:
            return

        if index is not None and 0 <= index <= len(self.files_list)-1:
            self.current_index = index
        self.actionPrior_image.setEnabled(self.current_index != 0)
        self.actionNext_image.setEnabled(self.current_index != len(self.files_list) - 1)

        pixmap = QtGui.QPixmap.fromImage(QtGui.QImage(os.path.join(self.files_root, self.files_list[self.current_index])))
        self.pixmap = pixmap
        self.scene.load_pixmap(pixmap)

        # update files dock
        self.listWidget_files.setCurrentRow(self.current_index)
        self.label_files.setText("{}/{}".format(self.current_index+1, len(self.files_list)))
        self.view.zoom_fit()

        filename = '.'.join(self.files_list[self.current_index].split('.')[:-1])
        self.xml_path = os.path.join(self.labels_root, filename + '.xml')

        self.load_anno()
        #
        self.actionCreate.setEnabled(True)
        self.actionZoom_in.setEnabled(True)
        self.actionZoom_out.setEnabled(True)
        self.actionFit_window.setEnabled(True)

    def load_anno(self):
        if not self.files_list:
            return

        if os.path.exists(self.xml_path):
            annos = Annotations()
            annos.load_annotation_from_xml(self.xml_path)

            for anno in annos.annotations:
                rect = Rect()
                self.scene.addItem(rect)
                color = '#6D6D6D'
                for cate, colo in self.category_tuples:
                    if cate == anno.category:
                        color = colo
                        break
                rect.create_from_xyxy(anno.category, anno.xmin, anno.ymin, anno.xmax, anno.ymax, anno.is_difficult, color)
                self.scene.rects.append(rect)
                self.dock_labels_add_label(rect)

    def prior_image(self):
        if self.current_index <= 0:
            QtWidgets.QMessageBox.warning(self, 'Warning', 'It`s the first one.',
                                          buttons=QtWidgets.QMessageBox.StandardButton.Yes)
            return
        if self.is_changed:
            button = QtWidgets.QMessageBox.warning(self, 'Warning', 'Unsaved! Are you sure want to continue?', buttons=QtWidgets.QMessageBox.StandardButton.Yes|QtWidgets.QMessageBox.StandardButton.No)
            if button == QtWidgets.QMessageBox.StandardButton.No:
                return
        self.current_index -= 1
        self.show_image()

    def next_image(self):
        if self.current_index >= len(self.files_list)-1:
            QtWidgets.QMessageBox.warning(self, 'Warning', 'It`s the last one.', buttons=QtWidgets.QMessageBox.StandardButton.Yes)
            return
        if self.is_changed:
            button = QtWidgets.QMessageBox.warning(self, 'Warning', 'Unsaved! Are you sure want to continue?', buttons=QtWidgets.QMessageBox.StandardButton.Yes|QtWidgets.QMessageBox.StandardButton.No)
            if button == QtWidgets.QMessageBox.StandardButton.No:
                return
        self.current_index += 1
        self.show_image()

    def jump_to(self):
        index = self.lineEdit_jump.text()
        if index:
            if not index.isdigit():
                if index in self.files_list:
                    index = self.files_list.index(index)+1
                else:
                    QtWidgets.QMessageBox.warning(self, 'Warning', 'Don`t exist image named: {}'.format(index))
                    self.lineEdit_jump.clear()
                    return
            index = int(index)-1
            if 0 <= index < len(self.files_list):
                self.show_image(index)
                self.lineEdit_jump.clear()
            else:
                QtWidgets.QMessageBox.warning(self, 'Warning', 'Index must be in [1, {}].'.format(len(self.files_list)))
                self.lineEdit_jump.clear()
                self.lineEdit_jump.clearFocus()
                return

    def dock_files_click(self):
        row = self.listWidget_files.currentRow()
        self.show_image(row)

    def label_item_widget_generate(self, rect):
        category = rect.category
        color = rect.color.name()
        is_difficult = rect.is_difficult
        item = QtWidgets.QListWidgetItem()
        item.setSizeHint(QtCore.QSize(200, 30))
        widget = QtWidgets.QWidget()

        layout = QtWidgets.QHBoxLayout()
        layout.setContentsMargins(9, 1, 9, 1)
        label_category = QtWidgets.QLabel()
        label_category.setText(category)

        button_checkbox = QtWidgets.QCheckBox()
        button_checkbox.setFixedWidth(20)
        button_checkbox.setChecked(rect.isVisible())
        button_checkbox.clicked.connect(rect.set_visible)
        button_checkbox.setObjectName('checkbox')

        label_color = QtWidgets.QLabel()
        label_color.setFixedWidth(10)
        label_color.setStyleSheet("background-color: {};".format(color))

        label_difficult =  QtWidgets.QLabel()
        label_difficult.setFixedWidth(3)
        if is_difficult:
            label_difficult.setStyleSheet("background-color: {};".format('#000000'))

        layout.addWidget(button_checkbox)
        layout.addWidget(label_color)
        layout.addWidget(label_category)
        layout.addWidget(label_difficult, alignment=QtCore.Qt.AlignmentFlag.AlignRight)

        widget.setLayout(layout)
        return item, widget

    def dock_labels_add_label(self, rect):
        item, widget = self.label_item_widget_generate(rect)
        self.listWidget_labels.addItem(item)
        self.listWidget_labels.setItemWidget(item, widget)

    def dock_labels_select_change(self):
        items = self.listWidget_labels.selectedItems()
        rows = [self.listWidget_labels.row(item) for item in items]
        for index, rect in enumerate(self.scene.rects):
            if index in rows:
                if not rect.isSelected():
                    rect.setSelected(True)
                    self.status_bar.showMessage("({:.0f}, {:.0f}, {:.0f}, {:.0f})".format(rect.vertexs[0].pos().x(),
                                                                                          rect.vertexs[0].pos().y(),
                                                                                          rect.vertexs[3].pos().x(),
                                                                                          rect.vertexs[3].pos().y()),
                                                3000)
            else:
                if rect.isSelected():
                    rect.setSelected(False)
        #
        rects_select = [rect.isSelected() for rect in self.scene.rects]
        any_selected = any(rects_select)
        self.actionEdit.setEnabled(any_selected)
        self.actionDelete.setEnabled(any_selected)

    def dock_labels_double_click(self):
        self.scene.edit_rect()

    def dock_labels_updata(self):
        self.listWidget_labels.clear()
        for rect in self.scene.rects:
            self.dock_labels_add_label(rect)

    def dock_label_set_selected(self, rect):
        index = self.scene.rects.index(rect)
        item = self.listWidget_labels.item(index)
        if rect.isSelected():
            if not item.isSelected():
                item.setSelected(True)
        else:
            if item.isSelected():
                item.setSelected(False)

    def label_visiable_checkbox_click(self, check_state):
        for index, rect in enumerate(self.scene.rects):
            rect.set_visible(check_state)
            item = self.listWidget_labels.item(index)
            widget = self.listWidget_labels.itemWidget(item)
            checkbox = widget.findChild(QtWidgets.QCheckBox, 'checkbox')
            checkbox.setChecked(check_state)
        self.checkBox_visiable.setChecked(check_state)

    def add_annotation_manually(self):
        if self.pixmap is None:
            return
        self.add_annotation_dialog.init()
        self.add_annotation_dialog.show()

    def setting(self):
        self.setting_dialog.init()
        self.setting_dialog.show()

    def save(self):
        if self.files_list and self.pixmap:
            annos = Annotations()
            annos.width = self.pixmap.width()
            annos.height = self.pixmap.height()
            annos.depth = self.pixmap.depth()
            annos.image_path = os.path.join(self.files_root, self.files_list[self.current_index])
            for rect in self.scene.rects:
                points = [v.pos() for v in rect.vertexs]
                xs = [p.x() for p in points]
                ys = [p.y() for p in points]

                xmin, ymin, xmax, ymax = min(xs), min(ys), max(xs), max(ys)
                anno = Annotation(rect.category, xmin, ymin, xmax, ymax, rect.is_difficult)
                annos.annotations.append(anno)
            annos.save_annotation_to_xml(self.xml_path)
            self.status_bar.showMessage('Save to {}'.format(self.xml_path), 2000)
            self.set_changed(False)

    def set_changed(self, changed:bool):
        if changed:
            self.is_changed =True
            self.setWindowTitle('*{}'.format(os.path.join(self.files_root, self.files_list[self.current_index])))
            self.actionSave.setEnabled(True)
        else:
            self.is_changed = False
            self.setWindowTitle('{}'.format(os.path.join(self.files_root, self.files_list[self.current_index])))
            self.actionSave.setEnabled(False)

    def connect(self):
        self.actionOpen_image_dir.triggered.connect(self.open_image_dir)
        self.actionOpen_label_dir.triggered.connect(self.open_label_dir)
        self.actionPrior_image.triggered.connect(self.prior_image)
        self.actionNext_image.triggered.connect(self.next_image)
        self.actionCreate.triggered.connect(self.scene.start_draw)
        self.actionEdit.triggered.connect(self.scene.edit_rect)
        self.actionDelete.triggered.connect(self.scene.delete_rect)
        self.actionSave.triggered.connect(self.save)
        self.actionCache.triggered.connect(self.scene.cache_draw)
        self.actionZoom_in.triggered.connect(self.view.zoom_in)
        self.actionZoom_out.triggered.connect(self.view.zoom_out)
        self.actionFit_window.triggered.connect(self.view.zoom_fit)

        self.actionSetting.triggered.connect(self.setting)
        self.actionShortcut.triggered.connect(self.shortcut_dialog.show)
        self.actionAbout.triggered.connect(self.about_dialog.show)
        self.actionExit.triggered.connect(self.close)

        self.actionChinese.triggered.connect(self.translate_to_chinese)
        self.actionEnglish.triggered.connect(self.translate_to_english)

        self.listWidget_files.doubleClicked.connect(self.dock_files_click)
        self.lineEdit_jump.returnPressed.connect(self.jump_to)
        self.listWidget_labels.itemSelectionChanged.connect(self.dock_labels_select_change)
        self.listWidget_labels.doubleClicked.connect(self.dock_labels_double_click)
        self.actionVisible.triggered.connect(self.label_visiable_checkbox_click)
        self.checkBox_visiable.clicked.connect(self.label_visiable_checkbox_click)
        self.pushButton_add_manually.clicked.connect(self.add_annotation_manually)

def main():
    app = QtWidgets.QApplication([''])
    mainwindow = MainWindow()
    mainwindow.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()




# -*- coding: utf-8 -*-
# @Author  : LG

from PyQt5 import QtCore, QtWidgets, QtGui
import typing

class Vertex(QtWidgets.QGraphicsPathItem):
    def __init__(self, rect, index):
        super(Vertex, self).__init__()
        self.rect = rect
        self.index = index
        self.color = QtGui.QColor('#ff0000')
        self.hover_size = 5
        self.nohover_size = 3

        self.nohover = QtGui.QPainterPath()
        self.nohover.addEllipse(
            QtCore.QRectF(-self.nohover_size // 2, -self.nohover_size // 2, self.nohover_size, self.nohover_size))
        self.hover = QtGui.QPainterPath()
        self.hover.addEllipse(
            QtCore.QRectF(-self.hover_size // 2, -self.hover_size // 2, self.hover_size, self.hover_size))

        self.setPath(self.nohover)
        self.setBrush(self.color)
        self.setPen(QtGui.QPen(self.color, 1))
        self.setCursor(QtGui.QCursor(QtCore.Qt.CursorShape.PointingHandCursor))

        self.setZValue(1e5)

        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, True)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsMovable, True)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemSendsGeometryChanges, True)
        self.setAcceptHoverEvents(True)

    def hoverEnterEvent(self, event: 'QtWidgets.QGraphicsSceneHoverEvent'):
        self.setPath(self.hover)
        return super(Vertex, self).hoverEnterEvent(event)

    def hoverLeaveEvent(self, event: 'QtWidgets.QGraphicsSceneHoverEvent'):
        self.setPath(self.nohover)
        return super(Vertex, self).hoverLeaveEvent(event)

    def itemChange(self, change: 'QtWidgets.QGraphicsItem.GraphicsItemChange', value: typing.Any):
        if change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemPositionChange and self.isEnabled():
            if not self.rect.is_completed:
                return super(Vertex, self).itemChange(change, value)
            value.setX(max(0, value.x()))
            value.setX(min(value.x(), self.scene().width()))
            value.setY(max(0, value.y()))
            value.setY(min(value.y(), self.scene().height()))
            self.rect.movePoint(self.index, value)

            self.scene().mainwindow.set_changed(True)
        return super(Vertex, self).itemChange(change, value)


class Rect(QtWidgets.QGraphicsRectItem):
    def __init__(self):
        super(Rect, self).__init__()
        self.points = []
        self.vertexs = []
        self.is_completed = False
        self.is_difficult = False
        self.color = QtGui.QColor('#000000')
        self.hover_alpha = 80
        self.nohover_alpha = 0

        self.setPen(QtGui.QPen(self.color, 1))
        self.setBrush(QtGui.QBrush(self.color, QtCore.Qt.BrushStyle.FDiagPattern))
        self.setZValue(1e4)

        self.setAcceptHoverEvents(True)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, True)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsMovable, True)
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemSendsGeometryChanges, True)

    def addPoint(self, pos):
        self.points.append(pos)

    def movePoint(self, index, pos):
        if index < 0 or index > len(self.points):
            return
        pos = self.mapFromScene(pos)
        if len(self.points) == 2:
            self.points[index] = pos
        elif len(self.points) == 4:
            if index == 0:
                self.points[1].setY(pos.y())
                self.points[2].setX(pos.x())
                self.moveVertex(1,self.mapToScene(self.points[1]))
                self.moveVertex(2,self.mapToScene(self.points[2]))
                self.points[0] = pos
            elif index == 1:
                self.points[0].setY(pos.y())
                self.points[3].setX(pos.x())
                self.moveVertex(0,self.mapToScene(self.points[0]))
                self.moveVertex(3,self.mapToScene(self.points[3]))
                self.points[1] = pos
            elif index == 2:
                self.points[0].setX(pos.x())
                self.points[3].setY(pos.y())
                self.moveVertex(0,self.mapToScene(self.points[0]))
                self.moveVertex(3,self.mapToScene(self.points[3]))
                self.points[2] = pos

            elif index == 3:
                self.points[1].setX(pos.x())
                self.points[2].setY(pos.y())
                self.moveVertex(1,self.mapToScene(self.points[1]))
                self.moveVertex(2,self.mapToScene(self.points[2]))
                self.points[3] = pos
        self.redraw()

    def moveVertex(self, index, pos):
        vertex = self.vertexs[index]
        vertex.setEnabled(False)
        vertex.setPos(pos)
        vertex.setEnabled(True)

    def removePoint(self, index):
        self.points.pop(index)
        self.redraw()

    def complete(self, category, color, difficult=False):
        self.category = category
        self.is_difficult = difficult
        self.color = QtGui.QColor(color)
        self.setPen(QtGui.QPen(self.color, 1, QtCore.Qt.PenStyle.SolidLine, QtCore.Qt.PenCapStyle.RoundCap))
        self.color.setAlpha(self.nohover_alpha)
        self.setBrush(self.color)

        lt, rb = self.points
        self.points.clear()
        lt, rt, lb, tb = lt, QtCore.QPointF(rb.x(), lt.y()), QtCore.QPointF(lt.x(), rb.y()), rb
        self.points = [lt, rt, lb, tb]
        for index, point in enumerate(self.points):
            vertex = Vertex(self, index)
            vertex.color = self.color
            vertex.setPos(point)
            self.vertexs.append(vertex)
            self.scene().addItem(vertex)
        self.redraw()
        self.is_completed = True
        
    def redraw(self):
        if len(self.points) == 2:
            xmin, ymin = min(self.points[0].x(), self.points[1].x()), min(self.points[0].y(), self.points[1].y())
            w, h = abs(self.points[0].x() - self.points[1].x()), abs(self.points[0].y() - self.points[1].y())
            self.setRect(xmin, ymin, w, h)

        if len(self.points) == 4:
            xmin = min(self.points[0].x(), self.points[1].x(), self.points[2].x(), self.points[3].x())
            xmax = max(self.points[0].x(), self.points[1].x(), self.points[2].x(), self.points[3].x())
            ymin = min(self.points[0].y(), self.points[1].y(), self.points[2].y(), self.points[3].y())
            ymax = max(self.points[0].y(), self.points[1].y(), self.points[2].y(), self.points[3].y())
            w, h = xmax - xmin, ymax - ymin
            self.setRect(xmin, ymin, w, h)

    def hoverEnterEvent(self, event: 'QtWidgets.QGraphicsSceneHoverEvent'):
        if self.is_completed:
            self.color.setAlpha(self.hover_alpha)
            self.setBrush(self.color)
        return super(Rect, self).hoverEnterEvent(event)

    def hoverLeaveEvent(self, event: 'QtWidgets.QGraphicsSceneHoverEvent'):
        if self.is_completed:
            self.color.setAlpha(self.nohover_alpha)
            self.setBrush(self.color)
        return super(Rect, self).hoverLeaveEvent(event)

    def itemChange(self, change: 'QtWidgets.QGraphicsItem.GraphicsItemChange', value: typing.Any):
        if self.is_completed:
            if change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemSelectedHasChanged and self.isEnabled():
                if self.isSelected():
                    color = QtGui.QColor('#00A0FF')
                    color.setAlpha(self.hover_alpha)
                    self.setBrush(color)
                else:
                    self.setBrush(self.color)

                self.scene().mainwindow.dock_label_set_selected(self)

            if change == QtWidgets.QGraphicsItem.GraphicsItemChange.ItemPositionChange:
                bias = value
                l, t, b, r = self.boundingRect().left(), self.boundingRect().top(), self.boundingRect().bottom(), self.boundingRect().right()
                if l + bias.x() < 0: bias.setX(-l)
                if r + bias.x() > self.scene().width(): bias.setX(self.scene().width() - r)
                if t + bias.y() < 0: bias.setY(-t)
                if b + bias.y() > self.scene().height(): bias.setY(self.scene().height() - b)

                for index, point in enumerate(self.points):
                    self.moveVertex(index, point + bias)
                self.scene().mainwindow.set_changed(True)
        return super(Rect, self).itemChange(change, value)

    def mouseDoubleClickEvent(self, event: 'QtWidgets.QGraphicsSceneMouseEvent'):
        if event.button() == QtCore.Qt.MouseButton.LeftButton:
            self.scene().edit_rect()

    def create_from_xyxy(self, category, xmin, ymin, xmax, ymax, is_difficult, color='#6D6D6D'):
        self.points = [QtCore.QPointF(xmin, ymin), QtCore.QPointF(xmax, ymax)]
        self.complete(category, color=color, difficult=is_difficult)

    def set_visible(self, visible):
        self.setVisible(visible)
        for vertex in self.vertexs:
            vertex.setVisible(visible)



# -*- coding: utf-8 -*-
# @Author  : LG

from setuptools import setup, find_packages


def get_version():
    from IDAT import __version__
    return __version__


setup(
    name="idat",
    version=get_version(),
    author="yatengLG",
    author_email="yatenglg@qq.com",
    description="Object Detection Annotation Tool.",
    long_description="Object Detection Annotation Tool.",
    url="https://github.com/yatengLG/IDAT",

    keywords=["pip", "idat", "object detection"],
    license="MIT Licence",

    packages=find_packages(),
    include_package_data=True,
    data_files=[
        ('IDAT', ['IDAT/default.yaml']),
        ('IDAT/ui', ['IDAT/ui/zh_CN.ts', 'IDAT/ui/en.ts', 'IDAT/ui/zh_CN.qm']),
    ],
    platforms="any",

    python_requires=">=3.6",                            # python 版本要求
    install_requires=[
        'imgviz',
        'pyqt5',
        'pyyaml'
    ],
    classifiers=[
        "Intended Audience :: Developers",  # 目标用户:开发者
        "Intended Audience :: Science/Research",  # 目标用户:学者
        'Development Status :: 5 - Production/Stable',
        "Natural Language :: Chinese (Simplified)",
        "Natural Language :: English",
        'License :: OSI Approved :: Apache Software License',
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
    ],
    entry_points={
        "console_scripts": [
            "idat=IDAT.main:main",
        ],
    },
)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/super/PycharmProjects/IDAT/shourcut_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog_Shortcut(object):
    def setupUi(self, Dialog_Shortcut):
        Dialog_Shortcut.setObjectName("Dialog_Shortcut")
        Dialog_Shortcut.resize(550, 250)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog_Shortcut)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget = QtWidgets.QWidget(Dialog_Shortcut)
        self.widget.setObjectName("widget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.widget_2 = QtWidgets.QWidget(self.widget)
        self.widget_2.setMaximumSize(QtCore.QSize(16777215, 40))
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.widget_2)
        self.label.setMaximumSize(QtCore.QSize(16777215, 50))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(20)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout_2.addWidget(self.widget_2)
        self.widget_3 = QtWidgets.QWidget(self.widget)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.widget_3.setFont(font)
        self.widget_3.setObjectName("widget_3")
        self.gridLayout = QtWidgets.QGridLayout(self.widget_3)
        self.gridLayout.setContentsMargins(18, -1, 18, -1)
        self.gridLayout.setObjectName("gridLayout")
        self.label_7 = QtWidgets.QLabel(self.widget_3)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 2, 2, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.widget_3)
        self.label_5.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setStyleSheet("color: rgb(133, 0, 0);")
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 1, 3, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.widget_3)
        self.label_14.setObjectName("label_14")
        self.gridLayout.addWidget(self.label_14, 3, 2, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.widget_3)
        self.label_11.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_11.setFont(font)
        self.label_11.setStyleSheet("color: rgb(133, 0, 0);")
        self.label_11.setObjectName("label_11")
        self.gridLayout.addWidget(self.label_11, 1, 1, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.widget_3)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 1, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.widget_3)
        self.label_6.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_6.setFont(font)
        self.label_6.setStyleSheet("color: rgb(133, 0, 0);")
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 2, 3, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.widget_3)
        self.label_12.setObjectName("label_12")
        self.gridLayout.addWidget(self.label_12, 3, 0, 1, 1)
        self.label_15 = QtWidgets.QLabel(self.widget_3)
        self.label_15.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_15.setFont(font)
        self.label_15.setStyleSheet("color: rgb(133, 0, 0);")
        self.label_15.setObjectName("label_15")
        self.gridLayout.addWidget(self.label_15, 3, 3, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.widget_3)
        self.label_8.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_8.setFont(font)
        self.label_8.setStyleSheet("color: rgb(133, 0, 0);")
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 2, 1, 1, 1)
        self.label_13 = QtWidgets.QLabel(self.widget_3)
        self.label_13.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_13.setFont(font)
        self.label_13.setStyleSheet("color: rgb(133, 0, 0);")
        self.label_13.setObjectName("label_13")
        self.gridLayout.addWidget(self.label_13, 3, 1, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.widget_3)
        self.label_9.setObjectName("label_9")
        self.gridLayout.addWidget(self.label_9, 2, 0, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.widget_3)
        self.label_10.setObjectName("label_10")
        self.gridLayout.addWidget(self.label_10, 1, 0, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.widget_3)
        self.label_18.setObjectName("label_18")
        self.gridLayout.addWidget(self.label_18, 0, 0, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.widget_3)
        self.label_19.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_19.setFont(font)
        self.label_19.setStyleSheet("color: rgb(133, 0, 0);")
        self.label_19.setObjectName("label_19")
        self.gridLayout.addWidget(self.label_19, 0, 1, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.widget_3)
        self.label_17.setObjectName("label_17")
        self.gridLayout.addWidget(self.label_17, 0, 2, 1, 1)
        self.label_16 = QtWidgets.QLabel(self.widget_3)
        self.label_16.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_16.setFont(font)
        self.label_16.setStyleSheet("color: rgb(133, 0, 0);")
        self.label_16.setObjectName("label_16")
        self.gridLayout.addWidget(self.label_16, 0, 3, 1, 1)
        self.verticalLayout_2.addWidget(self.widget_3)
        self.verticalLayout.addWidget(self.widget)

        self.retranslateUi(Dialog_Shortcut)
        QtCore.QMetaObject.connectSlotsByName(Dialog_Shortcut)

    def retranslateUi(self, Dialog_Shortcut):
        _translate = QtCore.QCoreApplication.translate
        Dialog_Shortcut.setWindowTitle(_translate("Dialog_Shortcut", "Shortcut"))
        self.label.setText(_translate("Dialog_Shortcut", "Shortcut"))
        self.label_7.setText(_translate("Dialog_Shortcut", "Delete rect"))
        self.label_5.setText(_translate("Dialog_Shortcut", "Esc"))
        self.label_14.setText(_translate("Dialog_Shortcut", "Zoom fit"))
        self.label_11.setText(_translate("Dialog_Shortcut", "C"))
        self.label_4.setText(_translate("Dialog_Shortcut", "Cache create"))
        self.label_6.setText(_translate("Dialog_Shortcut", "Del"))
        self.label_12.setText(_translate("Dialog_Shortcut", "Save annotation"))
        self.label_15.setText(_translate("Dialog_Shortcut", "F"))
        self.label_8.setText(_translate("Dialog_Shortcut", "E"))
        self.label_13.setText(_translate("Dialog_Shortcut", "S"))
        self.label_9.setText(_translate("Dialog_Shortcut", "Edit rect"))
        self.label_10.setText(_translate("Dialog_Shortcut", "Create rect"))
        self.label_18.setText(_translate("Dialog_Shortcut", "Prior image"))
        self.label_19.setText(_translate("Dialog_Shortcut", "A"))
        self.label_17.setText(_translate("Dialog_Shortcut", "Next image"))
        self.label_16.setText(_translate("Dialog_Shortcut", "D"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/super/PycharmProjects/IDAT/setting_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog_Setting(object):
    def setupUi(self, Dialog_Setting):
        Dialog_Setting.setObjectName("Dialog_Setting")
        Dialog_Setting.resize(558, 340)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        Dialog_Setting.setFont(font)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Dialog_Setting)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(Dialog_Setting)
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.listWidget_labels = QtWidgets.QListWidget(Dialog_Setting)
        self.listWidget_labels.setObjectName("listWidget_labels")
        self.verticalLayout.addWidget(self.listWidget_labels)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(6)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lineEdit_label = QtWidgets.QLineEdit(Dialog_Setting)
        self.lineEdit_label.setMinimumSize(QtCore.QSize(150, 0))
        self.lineEdit_label.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lineEdit_label.setReadOnly(False)
        self.lineEdit_label.setObjectName("lineEdit_label")
        self.horizontalLayout.addWidget(self.lineEdit_label)
        self.pushButton_color = QtWidgets.QPushButton(Dialog_Setting)
        self.pushButton_color.setMinimumSize(QtCore.QSize(30, 0))
        self.pushButton_color.setMaximumSize(QtCore.QSize(30, 16777215))
        self.pushButton_color.setStyleSheet("background-color: rgb(0, 0, 0);")
        self.pushButton_color.setText("")
        self.pushButton_color.setObjectName("pushButton_color")
        self.horizontalLayout.addWidget(self.pushButton_color)
        self.pushButton_add = QtWidgets.QPushButton(Dialog_Setting)
        self.pushButton_add.setMinimumSize(QtCore.QSize(150, 0))
        self.pushButton_add.setMaximumSize(QtCore.QSize(150, 16777215))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/书签_bookmark-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_add.setIcon(icon)
        self.pushButton_add.setAutoDefault(True)
        self.pushButton_add.setDefault(False)
        self.pushButton_add.setFlat(False)
        self.pushButton_add.setObjectName("pushButton_add")
        self.horizontalLayout.addWidget(self.pushButton_add)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.pushButton_import = QtWidgets.QPushButton(Dialog_Setting)
        self.pushButton_import.setMinimumSize(QtCore.QSize(80, 0))
        self.pushButton_import.setMaximumSize(QtCore.QSize(80, 16777215))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/icons/传入3_afferent-three.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_import.setIcon(icon1)
        self.pushButton_import.setObjectName("pushButton_import")
        self.horizontalLayout_2.addWidget(self.pushButton_import)
        self.pushButton_export = QtWidgets.QPushButton(Dialog_Setting)
        self.pushButton_export.setMinimumSize(QtCore.QSize(80, 0))
        self.pushButton_export.setMaximumSize(QtCore.QSize(80, 16777215))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/icons/传出3_efferent-three.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_export.setIcon(icon2)
        self.pushButton_export.setObjectName("pushButton_export")
        self.horizontalLayout_2.addWidget(self.pushButton_export)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.pushButton_cache = QtWidgets.QPushButton(Dialog_Setting)
        self.pushButton_cache.setMinimumSize(QtCore.QSize(80, 0))
        self.pushButton_cache.setMaximumSize(QtCore.QSize(80, 16777215))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/icons/icons/关闭_close-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_cache.setIcon(icon3)
        self.pushButton_cache.setObjectName("pushButton_cache")
        self.horizontalLayout_2.addWidget(self.pushButton_cache)
        self.pushButton_apply = QtWidgets.QPushButton(Dialog_Setting)
        self.pushButton_apply.setMinimumSize(QtCore.QSize(80, 0))
        self.pushButton_apply.setMaximumSize(QtCore.QSize(80, 16777215))
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/icons/icons/校验_check-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_apply.setIcon(icon4)
        self.pushButton_apply.setObjectName("pushButton_apply")
        self.horizontalLayout_2.addWidget(self.pushButton_apply)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.retranslateUi(Dialog_Setting)
        QtCore.QMetaObject.connectSlotsByName(Dialog_Setting)

    def retranslateUi(self, Dialog_Setting):
        _translate = QtCore.QCoreApplication.translate
        Dialog_Setting.setWindowTitle(_translate("Dialog_Setting", "Setting"))
        self.label.setText(_translate("Dialog_Setting", "Labels"))
        self.pushButton_add.setText(_translate("Dialog_Setting", "Add New Label"))
        self.pushButton_import.setText(_translate("Dialog_Setting", "Import"))
        self.pushButton_export.setText(_translate("Dialog_Setting", "Export"))
        self.pushButton_cache.setText(_translate("Dialog_Setting", "Cache"))
        self.pushButton_apply.setText(_translate("Dialog_Setting", "Apply"))



# -*- coding: utf-8 -*-
# @Author  : LG

from typing import List
from xml.etree import ElementTree as ET
import os


class Annotation(object):
    def __init__(self, category, xmin, ymin, xmax, ymax, is_difficult=False):
        self.category = category
        self.xmin = int(xmin) if isinstance(xmin, str) else xmin
        self.ymin = int(ymin) if isinstance(ymin, str) else ymin
        self.xmax = int(xmax) if isinstance(xmax, str) else xmax
        self.ymax = int(ymax) if isinstance(ymax, str) else ymax

        if is_difficult is not None:
            if isinstance(is_difficult, str):
                is_difficult = False if is_difficult == '0' else True
            if isinstance(is_difficult, int):
                is_difficult = False if is_difficult == 0 else True

        self.is_difficult = is_difficult


class Annotations(object):
    def __init__(self):
        self.annotations: List[Annotation] = []
        self.width:int = None
        self.height:int = None
        self.depth:int = None
        self.image_path:str = None

    def load_annotation_from_xml(self, xml_path):
        tree = ET.parse(xml_path)
        root= tree.getroot()

        size = root.find('size')
        width = size.find('width')
        height = size.find('height')
        depth = size.find('depth')
        self.width = width
        self.height = height
        self.depth = depth

        objs = root.findall('object')
        for obj in objs:
            name = obj.find('name').text
            difficult = obj.find('difficult').text
            bndbox = obj.find('bndbox')
            xmin = bndbox.find('xmin').text
            ymin = bndbox.find('ymin').text
            xmax = bndbox.find('xmax').text
            ymax = bndbox.find('ymax').text
            annotation = Annotation(name, xmin, ymin, xmax, ymax, difficult)
            self.annotations.append(annotation)

    def save_annotation_to_xml(self, xml_path):
        image_root, image_name = os.path.split(self.image_path)
        annotation = ET.Element('annotation')
        tree = ET.ElementTree(annotation)
        folder = ET.Element('folder')
        folder.text = '{}'.format(image_root)
        annotation.append(folder)

        filename = ET.Element('filename')
        filename.text = '{}'.format(image_name)
        annotation.append(filename)

        explain = ET.Element('explain')
        explain.text = '{}'.format('Build by IDAT v1.0')
        annotation.append(explain)

        size = ET.Element('size')
        width = ET.Element('width')
        width.text = '{}'.format(self.width)
        size.append(width)

        height = ET.Element('height')
        height.text = '{}'.format(self.height)
        size.append(height)

        depth = ET.Element('depth')
        depth.text = '{}'.format(self.depth)
        size.append(depth)
        annotation.append(size)

        for anno in self.annotations:

            object = ET.Element('object')
            name = ET.Element('name')
            name.text = anno.category
            object.append(name)

            pose = ET.Element('pose')
            pose.text = 'Unspecified'
            object.append(pose)

            truncated = ET.Element('truncated')
            truncated.text = '0'
            object.append(truncated)

            difficult = ET.Element('difficult')
            difficult.text = '1' if anno.is_difficult else '0'
            object.append(difficult)

            bndbox = ET.Element('bndbox')
            xmin = ET.Element('xmin')
            xmin.text = '{}'.format(int(anno.xmin))
            bndbox.append(xmin)

            ymin = ET.Element('ymin')
            ymin.text = '{}'.format(int(anno.ymin))
            bndbox.append(ymin)

            xmax = ET.Element('xmax')
            xmax.text = '{}'.format(int(anno.xmax))
            bndbox.append(xmax)

            ymax = ET.Element('ymax')
            ymax.text = '{}'.format(int(anno.ymax))
            bndbox.append(ymax)
            object.append(bndbox)
            annotation.append(object)

        # tree.write(xml_path, encoding='utf-8', xml_declaration=True)
        xml_string = ET.tostring(annotation, encoding='utf-8', method='xml')
        print(xml_string)
        from xml.dom import minidom
        xml_pretty = minidom.parseString(xml_string).toprettyxml(indent='    ')
        # print(xml_pretty)
        with open(xml_path, 'w') as f:
            f.write(xml_pretty)
        return True
